# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
import datetime
import math

import pymongo
from pyramid.view import view_config

from charmworld import cached_view_config, utils
from charmworld.models import (
    CharmSource
)
from charmworld.views.helpers import format_proof


# 3m
REPORT_CACHE = 180

def queue_latency_stats(queue):
    """Calculate the average, min, and max latency times for a review queue."""
    unreviewed_time = lambda x: datetime.datetime.now() - x['date_created']
    time_queue = [unreviewed_time(q) for q in queue if
        q['status'] in ['Needs review', 'New']]
    
    # It's nuts that we have to do this, but just using sum throws an error as
    # there's evidently an implicit int somehow.
    max_time = min_time = time_queue[0]
    summed_time = time_queue[0]
    for t in time_queue[1:]:
        if t > max_time:
            max_time = t
        if t < min_time:
            min_time = t
        summed_time += t
    average_time = summed_time/len(time_queue)
    return {
        "min": min_time,
        "max": max_time,
        "mean": average_time}


@cached_view_config(
    route_name="tools",
    renderer="charmworld:templates/tools.pt")
def tools(request):
    return {}


@view_config(
    route_name="tools-core-review",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/review.pt")
def core_review(request):
    # This view's data in a separate cron job than ingest
    queue = list(
        request.db.core_review_queue.find(
            sort=[('date_modified', pymongo.ASCENDING)]))
    return {'queue': queue, 'age_format': utils.pretty_timedelta}

@view_config(
    route_name="tools-review",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/review.pt")
def charm_review(request):
    # This view's data in a separate cron job than ingest
    queue = list(
        request.db.review_queue.find(
            sort=[('date_modified', pymongo.ASCENDING)]))
    stats = queue_latency_stats(queue)
    return {
        'queue': queue,
        'age_format': utils.pretty_timedelta,
        'max': utils.prettify_timedelta(stats['max']),
        'mean': utils.prettify_timedelta(stats['mean']),
        'min': utils.prettify_timedelta(stats['min'])}


@view_config(
    route_name="tools-store-missing",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/store-missing.pt")
def charm_store_missing(request):
    queue = list(CharmSource.from_request(request).find_mia_charms())
    return {'queue': queue, 'age_format': utils.pretty_timedelta}


@view_config(
    route_name="tools-proof-errors",
    http_cache=REPORT_CACHE,
    renderer="charmworld:templates/charm-proof-errors.pt")
def proof_errors(request):
    sub_only = bool(request.GET.get('sub'))
    source = CharmSource.from_request(request)
    charms = list(source.find_proof_error_charms(sub_only))

    return {
        'charms': charms,
        'format_proof': format_proof,
        'sub_only': sub_only,
    }
